/*
Class-name: EmptyFemObject.java. Object-Plugin for the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

import interfaces.CadObject;
import interfaces.Vertex;

import java.awt.Color;
import java.awt.Shape;
import java.util.Vector;

//import Vertex;

public class EmptyCadObject implements CadObject
{
	public static final int EXTRUDING_X = 0;	// Extruding in X-Direction
	public static final int EXTRUDING_Y = 1;	// Extruding in Y-Direction
	public static final int EXTRUDING_Z = 2;	// Extruding in Z-Direction (default)
	
	public static final int BUILT_IN_PANEL_0 = 0;	// Extruding in X-Direction
	public static final int BUILT_IN_PANEL_1 = 1;	// Extruding in Y-Direction
	public static final int BUILT_IN_PANEL_2 = 2;	// Extruding in Z-Direction (default)

	// Constants for 2D-objects
	public static final int GEOMETRY_2D_LINE = 0;			// Geometrytyp is a simple line (default)
	public static final int GEOMETRY_2D_POLYGON = 1;		// Geometrytyp is a poly-line
	public static final int GEOMETRY_2D_RECTANGLE = 2;	// Geometrytyp is a rectangle
	public static final int GEOMETRY_2D_CIRCLE = 3;			// Geometrytyp is a circle
	public static final int GEOMETRY_2D_ELLIPSE = 4;		// Geometrytyp is a ellipse
	public static final int GEOMETRY_2D_TRIANGLE = 5;		// Geometrytyp is a triangle
	public static final int GEOMETRY_2D_SURFACE = 6;		// Geometrytyp is a surface
	
	// Constants for 3D-objects
	public static final int GEOMETRY_3D_SPHERE = 7;			// Geometrytyp is a sphere
	public static final int GEOMETRY_3D_CUBE = 8;			// Geometrytyp is a cube
	public static final int GEOMETRY_3D_BOX =9;				// Geometrytyp is a box
	public static final int GEOMETRY_3D_CYLINDER = 10;		// Geometrytyp is a cylinder
	public static final int GEOMETRY_3D_CONE = 11;			// Geometrytyp is a cone	
	
	// CAD-Object Membervariables
	protected String 		name = "Blank";

	protected int			geometryTyp = GEOMETRY_2D_LINE;
	protected int 			layerID = 0;
	protected int 			objectId = 0;
	protected int 			numberOfPoints = 2;
	protected int 			lineTyp = 0;
	protected int 			extrudingDirection = EXTRUDING_Z;
	protected int			builtInView = BUILT_IN_PANEL_0;

	protected boolean 		visible = true;
	protected boolean 		filled = false;
	protected boolean 		fillable = true;
	protected boolean 		inBuildProcess = true;	// true, if object-dimensions still changing ?!
	protected boolean 		newObject = true;		// true, if object still is in build process ?!
	protected boolean 		startPointsSetted = false;	// Testing, if the startpoint is setted
	protected boolean		rendered2D = false;

	protected double  		lineWidth = 1d;
	protected double 		transparency = 1d;
	protected double 		extrusionStart = 0d, extrusionStop = 0d;

	protected Color 		lineColor = Color.BLACK;
	protected Color 		fillColor = Color.YELLOW;

	protected Shape 		shape = null;
	protected Shape 		shapeXY = null;
	protected Shape 		shapeZY = null;
	protected Shape 		shapeXZ = null;
	
	
	protected Vector<Vertex> vertexList;

	// Helpful members
	protected int			currentPointNumber = 0;
	protected boolean		finiteObject = true;
	
	public EmptyCadObject()
	{
		setName("EmptyFemObject");
		setFillable(false);
		initEnvironment();
	}
	public EmptyCadObject(String name)
	{
		setName(name);
		setFillable(false);
		initEnvironment();
	}
	
	protected void initEnvironment()
	{
		if(numberOfPoints>0) {
			vertexList = new Vector<Vertex>(numberOfPoints);
			for(int i = 0; i<numberOfPoints; i++) {
				vertexList.add(new Vertex(0d,0d,0d));
			}
		} else
			vertexList = new Vector<Vertex>();
			
			if(numberOfPoints==0)
				finiteObject = false;
	}
	
	public void setObjectPoints(int currentPoint, double x1, double y1, double z1, double x2, double y2, double z2, double scale)
	{
		this.currentPointNumber = currentPoint;
		
		if(finiteObject) {	// Setting the objects-points by known number of points
			
			// Setting the first point ...
			if(!startPointsSetted) {
				vertexList.set(0, new Vertex(x1 / scale, y1 / scale, z1 / scale));
				
				for(int i = 0; i<numberOfPoints; i++)
					vertexList.set(i, new Vertex(x1 / scale, y1 / scale, z1 / scale));
	
				startPointsSetted = true;
			}
			// know setting the following points
			else {
				vertexList.set(currentPointNumber, new Vertex(x2 / scale, y2 / scale, z2 / scale));
				if(currentPointNumber<numberOfPoints-1)
					vertexList.set(currentPointNumber+1,new Vertex(x2 / scale, y2 / scale, z2 / scale));
			}
		}
		else {	// Setting the objects-points by unknown number of points
			if(!startPointsSetted) {
				vertexList.setSize(2);
				vertexList.set(0, new Vertex(x1 / scale, y1 / scale, z1 / scale));
				vertexList.set(1, new Vertex(x1 / scale, y1 / scale, z1 / scale));
				
				startPointsSetted = true;
			}
			else {
				// resizing the vertexList-vector
				vertexList.setSize(currentPoint+1);
				vertexList.set(currentPoint, new Vertex(x2 / scale, y2 / scale, z2 / scale));
			}
		}
	}
	
	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return visible;
	}
	public Color getFillColor() {
		return fillColor;
	}
	public void setFillColor(Color fillColor) {
		this.fillColor = fillColor;
	}
	public boolean isFilled() {
		return filled;
	}
	public void setFilled(boolean filled) {
		this.filled = filled;
	}
	public Color getLineColor() {
		return lineColor;
	}
	public void setLineColor(Color lineColor) {
		this.lineColor = lineColor;
	}

	/**
	 * @param panelID Parameter is [0-2]. Shows, which panel-view is activ. Needed for the front-, top- or side-view.
	 * @param scale Scalingfactor for zooming
	 * @param mode If mode is 'true', FemObject is still in build-process, if mode is 'false' it is built 
	 */
	public Shape getShape(int panelID) {
		return getShape(panelID, 1d); // initShape(panelID, 1d);
	}
	/**
	 * @param panelID Parameter is [0-2]. Shows, which panel-view is activ. Needed for the front-, top- or side-view.
	 * @param scale Scalingfactor for zooming
	 * @param mode If mode is 'true', FemObject is still in build-process, if mode is 'false' it is built 
	 */
	public Shape getShape(int panelID, double scale)
	{
//		System.out.println("objectId "+objectId+" - rendered2D: "+rendered2D+" - inBuildProcess: "+inBuildProcess); 

		if(rendered2D && !inBuildProcess) {
//			System.out.println("Neu Rendern NICHT ntig");

			if(panelID==0) return shapeXY;
			else if(panelID==1) return shapeZY;
			else return shapeXZ;
		}
		else {
//			System.out.println("Neu Rendern");

			rendered2D = true;
			return initShape(panelID, scale);
		}
	}

	public void setShape(Shape shape) {
		this.shape = shape;
	}
	public boolean isInBuildProcess() {
		return inBuildProcess;
	}

	public void setInBuildProcess(boolean inBuildProcess) {
		this.inBuildProcess = inBuildProcess;
	}

	public boolean isNewObject() {
		return newObject;
	}

	public void setNewObject(boolean newObject) {
		this.newObject = newObject;
	}
	
	public int getObjectId() {
		return objectId;
	}
	public void setObjectId(int objectId) {
		this.objectId = objectId;
	}
	
	public int getLayerID() {
		return layerID;
	}
	public void setLayerID(int layerID) {
		this.layerID = layerID;
	}

	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}

	public double getLineWidth() {
		return lineWidth;
	}
	public void setLineWidth(double lineWidth) {
		this.lineWidth = lineWidth;
	}

	public boolean isFillable() {
		return fillable;
	}
	public void setFillable(boolean fillable) {
		this.fillable = fillable;
	}

	public int getNumberOfPoints() {
		return numberOfPoints;
	}
	public void setNumberOfPoints(int numberOfPoints) {
		this.numberOfPoints = numberOfPoints;
	}
	public Vector<Vertex> getVertexList() {
		return vertexList;
	}
	public void setVertexList(Vector<Vertex> vertexList) {
		this.vertexList = vertexList;
	}
	public int getLineTyp() {
		return lineTyp;
	}
	public void setLineTyp(int lineTyp) {
		this.lineTyp = lineTyp;
	}
	public double getTransparency() {
		return transparency;
	}
	public void setTransparency(double transparency) {
		this.transparency = transparency;
	}
	public int getExtrudingDirection() {
		return extrudingDirection;
	}
	public void setExtrudingDirection(int extrudingDirection) {
		this.extrudingDirection = extrudingDirection;
	}
	public int getGeometryTyp() {
		return geometryTyp;
	}
	public void setGeometryTyp(int geometryTyp) {
		this.geometryTyp = geometryTyp;
	}
	public String toString()
	{
		return "[name="+name+", geometryTyp="+geometryTyp+", objectId="+objectId+"]";
	}
	public int getCurrentPointNumber() {
		return currentPointNumber;
	}
	public void setCurrentPointNumber(int currentPointNumber) {
		this.currentPointNumber = currentPointNumber;
	}
	public double getExtrusionStart() {
		return extrusionStart;
	}
	public void setExtrusionStart(double extrusionStart) {
		this.extrusionStart = extrusionStart;
	}
	public double getExtrusionStop() {
		return extrusionStop;
	}
	public void setExtrusionStop(double extrusionStop) {
		this.extrusionStop = extrusionStop;
	}
	public boolean isRendered2D() {
		return rendered2D;
	}
	public void setRendered2D(boolean rendered2D) {
		this.rendered2D = rendered2D;
	}
	public Shape initShape(int panelID, double scale) {
		return shape;
	}
	public Shape getShapeXY() {
		return shapeXY;
	}
	public Shape getShapeXZ() {
		return shapeXZ;
	}
	public Shape getShapeZY() {
		return shapeZY;
	}
	public int getBuiltInView() {
		return builtInView;
	}
	public void setBuiltInView(int builtInView) {
		this.builtInView = builtInView;
	}
	
	public void finalizeObject(int quadrant)
	{
		// TODO Auto-generated method stub
		
	}

}
