/*
Class-name: MeshingTest.java, part of the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a (soon) integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

import interfaces.CadObject;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

public class MeshingTest extends EmptyExportImportObject implements ActionListener
{
	private JFrame frame = null;
	private MyPanel myPointGridPanel = null;
	private MyPanel myShapeGridPanel = null;
	private JPanel northPanel = null;
	private JPanel centerPanel = null;
	private JTabbedPane tabbedPane = null;
	private int defaultDimension = 640;
	private float scale = 1f;
	
	public MeshingTest()
	{
		frame = new JFrame();
		frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		frame.setAlwaysOnTop(true);
		frame.setSize(defaultDimension, defaultDimension);
		
		northPanel = new JPanel();
		centerPanel = new JPanel();	centerPanel.setLayout(new GridLayout(1,1));
		myPointGridPanel = new MyPanel(0);
		myShapeGridPanel = new MyPanel(1);
		tabbedPane = new JTabbedPane();
		
		JButton updateButton = new JButton("Update object- & layerdata"); updateButton.setName("update"); updateButton.addActionListener(this); 
		northPanel.add(updateButton);
		updateButton = new JButton("+"); updateButton.setName("zoomIn"); updateButton.addActionListener(this); 
		northPanel.add(updateButton);
		updateButton = new JButton("-"); updateButton.setName("zoomOut"); updateButton.addActionListener(this); 
		northPanel.add(updateButton);
		
		tabbedPane.add("myShapeGridPanel", myShapeGridPanel);
		tabbedPane.add("myPointGridPanel", myPointGridPanel);
		
		centerPanel.add(tabbedPane);
		
		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add(northPanel, BorderLayout.NORTH);
		frame.getContentPane().add(centerPanel, BorderLayout.CENTER);
	}
	
	public boolean isVisible()
	{
		return frame.isVisible();
	}

	public void setVisible(boolean visible)
	{
		frame.setVisible(visible);
	}

	public String getName()
	{
		return frame.getName();
	}

	public void setName(String name)
	{
		frame.setName(name);
		frame.setTitle("Simulator-Class: "+name);
	}

	public String toString()
	{
		return "class="+this.getClass().getCanonicalName();
		
	}
	
	public void actionPerformed(ActionEvent e)
	{
		JButton button = (JButton)e.getSource();
		System.out.println(button.getName());
		
		if(button.getName().equals("update")) {
			myPointGridPanel.repaint();
			myShapeGridPanel.repaint();
		} else if(button.getName().equals("zoomIn")) {
			scale *=2d;
			if(scale>8f)
				scale = 8f;

			myPointGridPanel.repaint();
			myShapeGridPanel.repaint();
			
		} else if(button.getName().equals("zoomOut")) {
			scale /=2d;
			if(scale<0.125f)
				scale =0.125f;
			
			myPointGridPanel.repaint();
			myShapeGridPanel.repaint();
			
		}
	}

	/**
	 * Paints the graphic-objects with a Grid based on click-Points or based on the minima and manima of the return shape
	 * @author Karsten
	 *
	 */
	class MyPanel extends JPanel
	{
		Vector<Double> xLines = new Vector<Double>();
		Vector<Double> yLines = new Vector<Double>();
		
		int gridMode = 0;

		public MyPanel(int gridMode)
		{
			this.gridMode = gridMode;
			
			this.setLayout(new BorderLayout());
			this.add(new JLabel("myPanel"));
		}
		
		public void paint(Graphics g)
		{
			super.paintComponent(g);
			
//			double scale = (double)Math.min(getWidth(), getHeight()) / (double)defaultDimension;
			
			Graphics2D g2 = (Graphics2D) g;
			
			g2.setStroke(new BasicStroke(1f / (float)scale));
			g2.scale(scale, scale);
			
//			System.out.println("cadObjects.size()="+cadObjects.size());
			
			if(cadObjects.size()>0) {
				xLines.clear();
				yLines.clear();
				
				if(gridMode==0)
					matchPointGridLines();
				else
					matchShapeGridLines();
				
				// painting the grid
				for(int j=0; j<xLines.size(); j++) {
					g2.setColor(Color.WHITE);
					g2.draw(new Line2D.Double(0d, yLines.get(j), getWidth(), yLines.get(j)));
					g2.draw(new Line2D.Double(xLines.get(j), 0d, xLines.get(j), getHeight()));
				}
				
				// painting the 2D/3D-graphic-object
				for(int i=0; i<cadObjects.size(); i++) {
					g2.setColor(Color.BLACK);			
					g2.draw(cadObjects.get(i).getShape(0));
				}
			}
		}
		
		/**
		 * Matching the click-points of the graphic-objects
		 */
		private void matchPointGridLines()
		{
			CadObject cadObject = null;
			
			for(int i=0; i<cadObjects.size(); i++) {
				cadObject = cadObjects.get(i);
				for(int j=0; j<cadObject.getVertexList().size(); j++) {
					xLines.add(cadObject.getVertexList().get(j).getX());
					yLines.add(cadObject.getVertexList().get(j).getY());
				}
			}
		}
		
		/**
		 * Matching the points of the minima and the manima of the return shape
		 */
		private void matchShapeGridLines()
		{
			CadObject cadObject = null;
			
			for(int i=0; i<cadObjects.size(); i++) {
				cadObject = cadObjects.get(i);
				xLines.add(cadObject.getShape(0).getBounds().getMinX());
				xLines.add(cadObject.getShape(0).getBounds().getMaxX());
				yLines.add(cadObject.getShape(0).getBounds().getMinY());
				yLines.add(cadObject.getShape(0).getBounds().getMaxY());
			}
		}
	}
}

