/*
Class-name: OspExportXML.java, part of the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a (soon) integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

//import interfaces.ExportObject;

import interfaces.CadObject;
import interfaces.Layer;
import interfaces.Vertex;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JFileChooser;

public class OspExportXML extends EmptyExportImportObject
{
	private String currentFilePath = null;
	
	public void exportObjects(ArrayList<CadObject> cadObjects, Vector<Layer> layers, String path)
	{
		System.out.println("exportObjects");
		
		String fileName = selectFile(path);
		
		if(fileName!=null) {
			StringBuffer string = new StringBuffer();
			CadObject swapObject;
			Layer swapLayer;
			
			string.append("<?xml version=\"" + "1.0\""+ " encoding=\""+ "ISO-8859-1\"" + "?>\n");
			string.append("<objects>\n");
			
			// generating the object-data
			string.append("\t<cadObjects>\n");
			for(int i=0; i<cadObjects.size(); i++) {
				swapObject = cadObjects.get(i);
				string.append("\t\t<object id=\""+i+"\">\n");
				string.append("\t\t\t<objectProperties geometryTyp=\""+swapObject.getGeometryTyp()+"\" className=\""+swapObject.getClass().getCanonicalName()+"\" name=\""+swapObject.getName()+"\"\n");
				string.append("\t\t\t\t objectId=\""+swapObject.getObjectId()+ "\" layerID=\""+swapObject.getLayerID()+"\" numberOfPoints=\""+swapObject.getNumberOfPoints()+"\"\n");
				string.append("\t\t\t\t visible=\""+swapObject.isVisible()+"\" filled=\""+swapObject.isFilled()+"\" fillable=\""+swapObject.isFillable()+"\"\n");
				string.append("\t\t\t\t lineWidth=\""+swapObject.getLineWidth()+"\" lineTyp=\""+swapObject.getLineTyp()+"\" transparency=\""+swapObject.getTransparency()+"\"\n");
				string.append("\t\t\t\t lineColor=\""+swapObject.getLineColor().getRGB()+"\" fillColor=\""+swapObject.getFillColor().getRGB()+"\" builtInView=\""+swapObject.getBuiltInView()+"\" >\n");
				string.append("\t\t\t</objectProperties>\n");
				string.append("\t\t\t<objectPoints>\n");
				string.append(vertexList2String(swapObject.getVertexList()));
				string.append("\t\t\t</objectPoints>\n");
				string.append("\t\t</object>\n");
			}
			string.append("\t</cadObjects>\n");
			// generating the object-data - END
			
			// generating the layer-data
			string.append("\n\t<layerObjects>\n");
			string.append("\t<!-- extrudingDirection-values: 0->x, 1->y, 2->z -->\n");
			for(int i=0; i<layers.size(); i++) {
				swapLayer = layers.get(i);
				string.append("\t\t<object id=\""+i+"\">\n");
				string.append("\t\t\t<defaultLayerProperties layerID=\""+swapLayer.getLayerID()+"\" name=\""+swapLayer.getName()+"\"\n");
				string.append("\t\t\tlineWidth=\""+swapLayer.getLineWidth()+"\" extrudingDirection=\""+swapLayer.getExtrudingDirection()+"\"\n");
				string.append("\t\t\textrusionStart=\""+swapLayer.getExtrusionStart()+"\" extrusionStop=\""+swapLayer.getExtrusionStop()+"\"\n");
				string.append("\t\t\tlineTyp=\""+swapLayer.getLineTyp()+"\" transparency=\""+swapLayer.getTransparency()+"\"\n");
				string.append("\t\t\tlineColor=\""+swapLayer.getLineColor().getRGB()+"\" fillColor=\""+swapLayer.getFillColor().getRGB()+"\" >\n");
				string.append("\t\t\t</defaultLayerProperties>\n");
				string.append("\t\t\t<addedLayerProperties>\n");
				string.append(addedLayerProperties2String(swapLayer)+"\n");
				string.append("\t\t\t</addedLayerProperties>\n");
				string.append("\t\t</object>\n");
			}
			string.append("\t</layerObjects>\n");
			// generating the layer-data - END

			string.append("</objects>\n");
			
			FileWriter export;
			try {
				export = new FileWriter(fileName);
				export.write(string.toString());
				export.close();
				System.out.println("Generated XML:\n"+string.toString()+"\n");
				System.out.println("File sucessfully exported!");	
			}
			catch (IOException e) { e.printStackTrace(); }
		} else
			System.out.println("File export wasn't sucessful!");	
	}
	
	public void importObjects(File file)
	{
		// TODO Auto-generated method stub
		
	}

	private String selectFile(String path)
	{
		JFileChooser fc = new JFileChooser();
		fc.setCurrentDirectory(new File(path));
		File file = null;
		String fileName = null;
		
		int retVal = fc.showSaveDialog( null );

		if(retVal==JFileChooser.APPROVE_OPTION ) {
			file = fc.getSelectedFile();
			fileName = file.getAbsolutePath(); // getName();
			currentFilePath = file.getPath();
			System.out.println( file.getAbsolutePath().replaceAll(file.getName(), "") );
		}
		else
			System.out.println( "Auswahl abgebrochen" );	
		
		return fileName;
	}
	
	/**
	 * Generating a string, with the setted points aof the 3d-objects 
	 * @param vertexList
	 * @return
	 */
	private String vertexList2String(Vector<Vertex> vertexList)
	{
		StringBuffer strBuf = new StringBuffer();
		Vertex vertex; 
		
		for(int i=0; i<vertexList.size(); i++) {
			vertex = vertexList.get(i);
			strBuf.append("\t\t\t\t<point id=\""+i+"\" value=\""+vertex.getX()+";"+vertex.getY()+";"+vertex.getZ()+"\" />\n");
		}
		
		if(strBuf.length()>0)
			return strBuf.toString();
		else
			return "";
	}

	/**
	 * Generating a string, with the names and properties of the dynamically added layer-properties 
	 * @param layer
	 * @return
	 */
	private String addedLayerProperties2String(Layer layer)
	{
		StringBuffer strBuf = new StringBuffer();
		Layer.LayerProperty lp; 

		for(int i=0; i<layer.getLayerProperties().size(); i++) {
			lp = layer.getLayerProperties().get(i);
			strBuf.append("\t\t\t\t<addedProperty id=\""+i+"\" name=\""+lp.getPropertyName()+"\" value=\""+lp.getPropertyValue()+"\" />\n");
		}
		
		if(strBuf.length()>0)
			return strBuf.toString();
		else
			return "";
	}

	public String getCurrentFilePath() {
		return currentFilePath;
	}
	public void setCurrentFilePath(String currentFilePath) {
		this.currentFilePath = currentFilePath;
	}
}
