/*
Class-name: OspImportXML.java, part of the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a (soon) integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

//import interfaces.ExportObject;

import interfaces.CadObject;
import interfaces.Layer;
import interfaces.Vertex;

import java.awt.Color;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class OspImportXML extends EmptyExportImportObject
{
	private String currentFilePath = null;
	
	private Vector<ImportObjectData> importObjects = new Vector<ImportObjectData>(); 

	boolean isObject, isLayer;
	
	String className, name;
	int geometryTyp, objectId, layerID, numberOfPoints, lineTyp, lineColor, fillColor, extrudingDirection, builtInView;
	double extrusionStart, extrusionStop, lineWidth, transparency, x, y, z;
	boolean visible, filled, fillable;
	
	Vector<Vertex> vertexList; //  = new Vector<Vertex>();

	public void importObjects(ArrayList<CadObject> cadObjects, Vector<Layer> layers, File file, String name, String path)
	{
		String fileName = selectFile(file.toString());
		System.out.println("importObjects");
		
		ImportObjectData importObject = null;
		DocumentBuilder db = null;
		org.w3c.dom.Document document = null; 	// Das XML-Dokument
		XPath xPath = null;						// XPath-Objekt, zum einlesen der Elemente und Attribute
		NodeList nodes = null;					// list nodes
		NodeList nodes2 = null;					// list nodes
		Node node = null;						// a single-node
		
		try {
			db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		
		try { document = db.parse(fileName); }
		catch (SAXException e) { e.printStackTrace(); }
		catch (IOException e) { e.printStackTrace();
		}

		xPath = XPathFactory.newInstance().newXPath();
			
		// parsing the XML-import
		try {

			// --- Reading the properties and the vertices of the CAD-objects ---
			nodes = (NodeList)xPath.evaluate("/objects/cadObjects/object", document, XPathConstants.NODESET);
			for(int i=0; i<nodes.getLength(); i++) {
				// Reading properties
				node = (Node)xPath.evaluate("./objectProperties", nodes.item(i), XPathConstants.NODE);
				
				// Reading the strings
				className = (String)(xPath.evaluate("./@className", node));
				name = (String)(xPath.evaluate("./@name", node));
				
				// reading all int-values
				geometryTyp = 	 new Integer(xPath.evaluate("./@geometryTyp", node));
				objectId = 		 new Integer(xPath.evaluate("./@objectId", node));
				layerID = 		 new Integer(xPath.evaluate("./@layerID", node));
				numberOfPoints = new Integer(xPath.evaluate("./@numberOfPoints", node));
				lineTyp = 		 new Integer(xPath.evaluate("./@lineTyp", node));
				lineColor = 	 new Integer(xPath.evaluate("./@lineColor", node));
				fillColor = 	 new Integer(xPath.evaluate("./@fillColor", node));
				builtInView = 	 new Integer(xPath.evaluate("./@builtInView", node));
				
				// Reading the double-values
				lineWidth = new Double(xPath.evaluate("./@lineWidth", node));
				transparency = new Double(xPath.evaluate("./@transparency", node));
				
				// And now reading the bools
				visible = new Boolean(xPath.evaluate("./@visible", node));
				filled = new Boolean(xPath.evaluate("./@filled", node));
				fillable = new Boolean(xPath.evaluate("./@fillable", node));
				// Reading properties - END
				
				// Reading vertices					
				nodes2 = (NodeList)xPath.evaluate("./objectPoints/point", nodes.item(i), XPathConstants.NODESET);
				String[] splitOfValue = null;
				Vertex vertex = null;
				vertexList = new Vector<Vertex>();
				for(int j=0; j<nodes2.getLength(); j++) {
					splitOfValue = ((String)(xPath.evaluate("./@value", nodes2.item(j)))).split(";");
					x = new Double(splitOfValue[0]);
					y = new Double(splitOfValue[1]);
					z = new Double(splitOfValue[2]);
					vertex = new Vertex(x, y, z);
					vertexList.add(vertex);
					
//					System.out.println("1. vertex: "+j+" -> x: "+vertex.getX()+" - y: "+vertex.getY()+" - z:"+vertex.getZ());
				}
				// Reading vertices - END					

				// Reading the properties of the CAD-objects - END ---
				
				importObject = new ImportObjectData();
				
				importObject.setClassName(className);
				importObject.setName(name);
				
				importObject.setGeometryTyp(geometryTyp);
				importObject.setObjectId(objectId);
				importObject.setLayerID(layerID);
				importObject.setNumberOfPoints(numberOfPoints);
				importObject.setLineTyp(lineTyp);
				importObject.setLineColor(lineColor);
				importObject.setFillColor(fillColor);
				importObject.setBuiltInView(builtInView);
				
				importObject.setLineWidth(lineWidth);
				importObject.setTransparency(transparency);
				
				importObject.setVisible(visible);
				importObject.setFilled(filled);
				importObject.setFillable(fillable);
				
				importObject.setVertexList(vertexList);
				
//				importObject.setRendered2D(true);
				
				importObjects.add(importObject);
				
				System.out.println(importObject.toString());
				
			}

			// And finally filling the CAD-Objects-vector with the data
			try { instancingCadObjects(cadObjects, name,  path); }
			catch (Exception e) { e.printStackTrace(); }
			
			// --- Reading the properties and the vertices of the CAD-objects - END ---

			
			// --- Reading the layerdata ---
			
			// First clearing ALL currently laxer-data
			layers.clear();

			Layer layer; // declarating a single Layer-instance
			
			nodes = (NodeList)xPath.evaluate("/objects/layerObjects/object", document, XPathConstants.NODESET);
			for(int k=0; k<nodes.getLength(); k++) {
//				System.out.println("default-objektknoten: "+k);
				
				node = (Node)xPath.evaluate("./defaultLayerProperties", nodes.item(k), XPathConstants.NODE);
				
				// now reading the strings,
				name = (String)(xPath.evaluate("./@name", node));

				// reading the integer-values				
				layerID = new Integer(xPath.evaluate("./@layerID", node));
				lineTyp = new Integer(xPath.evaluate("./@lineTyp", node));
				lineColor = new Integer(xPath.evaluate("./@lineColor", node));
				fillColor = new Integer(xPath.evaluate("./@fillColor", node));
				extrudingDirection = new Integer(xPath.evaluate("./@extrudingDirection", node));
				
				// and the double-values
				extrusionStart = new Double(xPath.evaluate("./@extrusionStart", node));
				extrusionStop = new Double(xPath.evaluate("./@extrusionStop", node));
				lineWidth = new Double(xPath.evaluate("./@lineWidth", node));
				transparency = new Double(xPath.evaluate("./@transparency", node));
				
				// Getting an instance of Layer
				layer = new Layer(name, layerID);
				
				// Now filling the layer with the values!
				layer.setLineTyp(lineTyp);
				layer.setLineColor(new Color(lineColor));
				layer.setFillColor(new Color(fillColor));
				layer.setExtrudingDirection(extrudingDirection);
				layer.setExtrusionStart(extrusionStart);
				layer.setExtrusionStop(extrusionStop);
				layer.setLineWidth(lineWidth);
				layer.setTransparency(transparency);

				// Reading and adding the additional properties
				nodes2 = (NodeList)xPath.evaluate("./addedLayerProperties/addedProperty", nodes.item(k), XPathConstants.NODESET);
				String propertyName = null;
				String propertyValue = null;
				for(int l=0; l<nodes2.getLength(); l++) {
					propertyName = (String)(xPath.evaluate("./@name", nodes2.item(l)));
					propertyValue = (String)(xPath.evaluate("./@value", nodes2.item(l)));
					layer.addNewProperty(propertyName, propertyValue);
//					System.out.println("propertyName: "+propertyName+" - propertyValue:"+propertyValue);
				}

				// Moving the imported layer to the GUIs layer-vector
				layers.add(layer);
			}
			// --- Reading the layerdata - END ---

		}
		catch (XPathExpressionException e) { e.printStackTrace(); }

	}

	private void instancingCadObjects(ArrayList<CadObject> cadObjects, String name, String path) throws Exception // throws MalformedURLException, ClassNotFoundException, InstantiationException, IllegalAccessException
	{
		// First deleting all currently existing CAD-objects
		cadObjects.clear();
		
		CadObject cadObject;
		ImportObjectData importObject;
		
	    // Importing CAD-objects
		for(int i=0; i<importObjects.size(); i++) {
			importObject = importObjects.get(i);
			
			// Creating a new CAD-object
		    URLClassLoader cl = new URLClassLoader(new URL[] { new File(path).toURL() });
			Class c = cl.loadClass(importObject.getClassName());
			cadObject = (CadObject)c.newInstance();
			
			// ... and now setting up the CAD-data 
			cadObject.setGeometryTyp(importObject.getGeometryTyp());
			cadObject.setObjectId(importObject.getObjectId());
			cadObject.setName(importObject.getName());
			cadObject.setLayerID(importObject.getLayerID());
			cadObject.setNumberOfPoints(importObject.getNumberOfPoints());
			cadObject.setVisible(importObject.isVisible());
			cadObject.setFilled(importObject.isFilled());
			cadObject.setFillable(importObject.isFillable());
			cadObject.setLineWidth(importObject.getLineWidth());
			cadObject.setLineTyp(importObject.getLineTyp());
			cadObject.setTransparency(importObject.getTransparency());
			cadObject.setLineColor(new Color(importObject.getLineColor()));
			cadObject.setFillColor(new Color(importObject.getFillColor()));
			cadObject.setVertexList(importObject.getVertexList());
			cadObject.setBuiltInView(importObject.getBuiltInView());
			
			// finally adding the CAD-object to the objects-vector
			cadObjects.add(cadObject);
		}

	}
	
	private String selectFile(String path)
	{
		JFileChooser fc = new JFileChooser();
		fc.setCurrentDirectory(new File(path));
		File file = null;
		String fileName = null;
		
		int retVal = fc.showOpenDialog( null );

		if(retVal==JFileChooser.APPROVE_OPTION ) {
			file = fc.getSelectedFile();
			fileName = file.getAbsolutePath();
			currentFilePath = file.getPath();
//			System.out.println( file.getAbsolutePath().replaceAll(file.getName(), "") );
		}
		else
			System.out.println( "Auswahl abgebrochen" );	
		
		return fileName;
	}
	
	public String getCurrentFilePath() {
		return currentFilePath;
	}
	public void setCurrentFilePath(String currentFilePath) {
		this.currentFilePath = currentFilePath;
	}
	
	protected class ImportObjectData
	{
		boolean isObject, isLayer;
		
		String className, name;
		int geometryTyp, objectId, layerID, numberOfPoints, lineTyp, lineColor, fillColor, builtInView;
		double lineWidth, transparency, x, y, z;
		boolean visible, filled, fillable; // , rendered2D;
		
		Vector<Vertex> vertexList = new Vector<Vertex>();

		public String getClassName() {
			return className;
		}

		public void setClassName(String className) {
			this.className = className;
		}

		public boolean isFillable() {
			return fillable;
		}

		public void setFillable(boolean fillable) {
			this.fillable = fillable;
		}

		public int getFillColor() {
			return fillColor;
		}

		public void setFillColor(int fillColor) {
			this.fillColor = fillColor;
		}

		public boolean isFilled() {
			return filled;
		}

		public void setFilled(boolean filled) {
			this.filled = filled;
		}

		public int getGeometryTyp() {
			return geometryTyp;
		}

		public void setGeometryTyp(int geometryTyp) {
			this.geometryTyp = geometryTyp;
		}

		public boolean isLayer() {
			return isLayer;
		}

		public void setLayer(boolean isLayer) {
			this.isLayer = isLayer;
		}

		public boolean isObject() {
			return isObject;
		}

		public void setObject(boolean isObject) {
			this.isObject = isObject;
		}

		public int getLayerID() {
			return layerID;
		}

		public void setLayerID(int layerID) {
			this.layerID = layerID;
		}

		public int getLineColor() {
			return lineColor;
		}

		public void setLineColor(int lineColor) {
			this.lineColor = lineColor;
		}

		public int getLineTyp() {
			return lineTyp;
		}

		public void setLineTyp(int lineTyp) {
			this.lineTyp = lineTyp;
		}

		public double getLineWidth() {
			return lineWidth;
		}

		public void setLineWidth(double lineWidth) {
			this.lineWidth = lineWidth;
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public int getNumberOfPoints() {
			return numberOfPoints;
		}

		public void setNumberOfPoints(int numberOfPoints) {
			this.numberOfPoints = numberOfPoints;
		}

		public int getObjectId() {
			return objectId;
		}

		public void setObjectId(int objectId) {
			this.objectId = objectId;
		}

		public double getTransparency() {
			return transparency;
		}

		public void setTransparency(double transparency) {
			this.transparency = transparency;
		}

		public Vector<Vertex> getVertexList() {
			return vertexList;
		}

		public void setVertexList(Vector<Vertex> vertexList) {
			this.vertexList = vertexList;
		}

		public boolean isVisible() {
			return visible;
		}

		public void setVisible(boolean visible) {
			this.visible = visible;
		}
		
		public String toString()
		{
			return className+" "+name+" "+geometryTyp+" "+objectId+" "+layerID+" "+numberOfPoints+" "+lineTyp+" "+lineColor+" "+fillColor+
			" "+lineWidth+" "+transparency+" "+visible+" "+filled+" "+fillable;
		}

		public int getBuiltInView() {
			return builtInView;
		}

		public void setBuiltInView(int builtInView) {
			this.builtInView = builtInView;
		}
	}
}
