/*
Class-name: SimpleEllipse.java. Object-Plugin for the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a (soon) integrated plugable simulation-interface

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

import java.awt.Shape;
import java.awt.geom.Arc2D;
import java.awt.geom.Line2D;

/**
 * SimpleLine: Paints a simple ellipse from point x1,y1 to x2,y2
 * @author Karsten Bettray
 */
public class PeaceOfCake extends EmptyCadObject // implements FemObject
{
	public PeaceOfCake()
	{
		setName("PeaceOfCake");
		setNumberOfPoints(3);
		setGeometryTyp(EmptyCadObject.GEOMETRY_2D_CIRCLE);
		setFillable(true);
		initEnvironment();
	}
	public PeaceOfCake(String name)
	{
		setName(name);
		setNumberOfPoints(3);
		setGeometryTyp(EmptyCadObject.GEOMETRY_2D_CIRCLE);
		setFillable(true);
		initEnvironment();
	}

	/**
	 * @param panelID Parameter is [0-2]. Shows, which panel-view is activ. Needed for the front-, top- or side-view. 
	 */
	public Shape initShape(int panelID, double scale)
	{	
		double x1, y1, x2, y2, x3, y3, radius, startDegr = 180, endDegr = 270;
		
		if(panelID==0) {
			x1 = (vertexList.get(0).getX() * scale); y1 = (vertexList.get(0).getY() * scale);
			x2 = (vertexList.get(1).getX() * scale); y2 = (vertexList.get(1).getY() * scale);
			x3 = (vertexList.get(2).getX() * scale); y3 = (vertexList.get(2).getY() * scale);
		} 
		else if(panelID==1) {
			x1 = (vertexList.get(0).getZ() * scale); y1 = (vertexList.get(0).getY() * scale);
			x2 = (vertexList.get(1).getZ() * scale); y2 = (vertexList.get(1).getY() * scale);
			x3 = (vertexList.get(2).getZ() * scale); y3 = (vertexList.get(2).getY() * scale);
		} 
		else {
			x1 = (vertexList.get(0).getX() * scale); y1 = (vertexList.get(0).getZ() * scale);
			x2 = (vertexList.get(1).getX() * scale); y2 = (vertexList.get(1).getZ() * scale);
			x3 = (vertexList.get(2).getX() * scale); y3 = (vertexList.get(2).getZ() * scale);
		}
		
		System.out.println("currentPointNumber: "+currentPointNumber);
		
		radius = Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1)); 
		
		startDegr = -((180d / Math.PI) * Math.atan2((y2-y1),(x2-x1)));
		
		endDegr = -((180d / Math.PI) * Math.atan2((y3-y1),(x3-x1))) - startDegr;

		System.out.println("startDegr: "+startDegr+" - "+endDegr);
		
		if(x1 <= x2 && y1 > y2) {
			
		}
		
//		if(x1 <= x2 && y1 <= y2) {
//			endDegr += 180d;
//		}
//		else if(x1 > x2 && y1 <= y2) {
//			endDegr = 360;
//		}
//		else if(x1 <= x2 && y1 > y2) {
//			endDegr = 360;
//		}
//		else { // x1 > x2 && y1 > y2
//			endDegr = 360;
//		}

		if(currentPointNumber==1)
			shape = new Line2D.Double(x1, y1, x2, y2);
		else
			shape = new Arc2D.Double(x1 - radius, y1 - radius, 2d*radius, 2d*radius, startDegr, endDegr, Arc2D.PIE);
		
		return shape;
	}
	
	
	public int getLineTyp() {
		return EmptyCadObject.GEOMETRY_2D_ELLIPSE;
	}
}
