/*
Class-name: PolyLine3Points.java. Object-Plugin for the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a (soon) integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

import java.awt.Color;
import java.awt.Shape;
import java.awt.geom.GeneralPath;

public class PolyLine3Points extends EmptyCadObject // implements FemObject
{
	GeneralPath gp = null;
	
	public PolyLine3Points()
	{
		setName("PolyLine3Points");
		setNumberOfPoints(3);
		setGeometryTyp(EmptyCadObject.GEOMETRY_2D_POLYGON);
		setFillable(true);
		setFillColor(Color.GREEN);
		initEnvironment();
	}
	public PolyLine3Points(String name)
	{
		setName(name);
		setNumberOfPoints(3);
		setGeometryTyp(EmptyCadObject.GEOMETRY_2D_POLYGON);
		setFillable(true);
		setFillColor(Color.GREEN);
		initEnvironment();
	}
	
	/**
	 * @param panelID Parameter is [0-2]. Shows, which panel-view is activ. Needed for the front-, top- or side-view.
	 * @param scale Scalingfactor for zooming
	 * @param mode If mode is 'true', FemObject is still in build-process, if mode is 'false' it is built 
	 */
	public Shape initShape(int panelID, double scale)
	{
		gp = new GeneralPath();
		gp.moveTo((float)(vertexList.get(0).getX() * scale), (float)(vertexList.get(0).getY() * scale));
		for(int i=1; i<vertexList.size(); i++) {
			gp.lineTo((float)(vertexList.get(i).getX() * scale), (float)(vertexList.get(i).getY() * scale));
		}
		shapeXY = gp;

		gp = new GeneralPath();
		gp.moveTo((float)(vertexList.get(0).getZ() * scale), (float)(vertexList.get(0).getY() * scale));
		for(int i=1; i<vertexList.size(); i++) {
			gp.lineTo((float)(vertexList.get(i).getZ() * scale), (float)(vertexList.get(i).getY() * scale));
		}
		shapeZY = gp;

		gp = new GeneralPath();
		gp.moveTo((float)(vertexList.get(0).getX() * scale), (float)(vertexList.get(0).getZ() * scale));
		for(int i=1; i<vertexList.size(); i++) {
			gp.lineTo((float)(vertexList.get(i).getX() * scale), (float)(vertexList.get(i).getZ() * scale));
		}
		shapeXZ = gp;
		
		if(panelID==0)
			shape = shapeXY;
		else if(panelID==1)
			shape = shapeZY;
		else
			shape = shapeXZ;
		
		return shape;
	}

	public int getLineTyp() {
		return EmptyCadObject.GEOMETRY_2D_POLYGON;
	}
}
