/*
Class-name: ReadXML.java. Object-Plugin for the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a integrated plugable simulation-interface 

Copyright (C) 2006 Kay Dnzer	

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

import interfaces.CadObject;
import interfaces.Layer;
import interfaces.Vertex;

import java.awt.Color;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.vecmath.Vector3f;

import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;


public class ReadXML extends EmptyExportImportObject{

	private String currentFilePath = null;
	private static int currentID = 0;

	public void importObjects(ArrayList<CadObject> cadObjects, Vector<Layer> layers, File file, String name, String path)
	{
		SAXBuilder builder=null;
		org.jdom.Document d = null;
		Element root = null;
		
		String fileName = selectFile(file.toString());

		System.out.println("Beginne Ladevorgang");
		currentID = 0;


// Datei mit JDom ffnen:
		builder = new SAXBuilder();
		try {
			d = builder.build(fileName);
		} catch (JDOMException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		root = d.getRootElement();
	
// *** Layerset einlesen ***
		layers.clear();
		
		Layer layer = new Layer();
		layer.setName("Default-Layer");
		layer.setLayerID(0);
		layers.add(layer);
		
		try {
			root = d.getRootElement().getChild("layerset");
			List children = root.getChildren();
			Iterator iterator = children.iterator();
			
			while (iterator.hasNext()) {
				
				Element child = (Element)iterator.next();
				if (child.getName().equals("layer")) {
			    	   layers.add(readLayer(child));
				}
			}
		    System.out.println("Anzahl Layer: "+layers.size());
		} catch (NullPointerException e) {System.out.println("Fehler beim Lesen der Layer");}
// *** Layerset einlesen Ende ***
		
// *** Boxset einlesen ***
		cadObjects.clear();
		try {
			root = d.getRootElement().getChild("boxset");
			List children = root.getChildren();
			Iterator iterator = children.iterator();
			while (iterator.hasNext()) {
				Element child = (Element)iterator.next();
				if (child.getName().equals("box")) {
			    		readBox(cadObjects, child, path);
		       }
			}
		} catch (Exception e) {System.out.println("Fehler beim Lesen der Boxen: "+e);}
// *** Boxset einlesen Ende ***		

// *** Polyset einlesen ***
		try {
			root = d.getRootElement().getChild("polyset");
			List children = root.getChildren();
			Iterator iterator = children.iterator();

			while (iterator.hasNext()) {
				
				Element child = (Element)iterator.next();
				if (child.getName().equals("poly")&&child.getAttribute("structure").getValue().toString().equals("rotated")) {
			    	   readSphere(cadObjects,child,path);
				}  else
				if (child.getName().equals("poly")&&child.getAttribute("structure").getValue().toString().equals("extruded")) {
					boolean err=false;
					try {													//diese Methode zu berprfen,
						child.getChild("radius").getAttribute("value").getValue().toString();//ob es sich um einen Zylinder
					} catch (NullPointerException e) {err=true;}			//oder ein ClosedPoly handelt
					if (!err) {												//ist nicht sehr glcklich. Ich
						readCylinder(cadObjects,child,path);
					} else {
						err=false;
						try {
							child.getChild("pointset").getAttribute("qty").getValue().toString();
						} catch (NullPointerException e) {err=true;}
						if (!err) {
							readClosedPoly(cadObjects,layers,child,path);
						}
					}
				}  else
				if (child.getName().equals("poly")&&child.getAttribute("structure").getValue().toString().equals("linear")) {
					readLinPoly(cadObjects,child,path);
				}
			}
		} catch (Exception e) {System.out.println("Fehler beim Lesen der Polys: "+e);};
// *** Polyset einlesen Ende ***		

// *** Surfacesets einlesen ***
		try {
			root = d.getRootElement().getChild("surfaceset");
			List children = root.getChildren();
			Iterator iterator = children.iterator();
			while (iterator.hasNext()) {
				
				Element child = (Element)iterator.next();
				if (child.getName().equals("surface")) {
			    	   readSurface(cadObjects,child,path);
				}  
			}
		} catch (Exception e) {System.out.println("Fehler beim Lesen der Surfaces: "+e);};
// *** Surfacesets einlesen Ende***

	}	

	public static Layer readLayer (Element child) {
		Layer obj = new Layer();

		String txt = child.getAttribute("id").getValue().toString();
		obj.setLayerID(new Integer(txt));
		
		try {
			txt = child.getAttribute("name").getValue().toString();
			obj.setName(txt);
		} catch (Exception e) {
			obj.setName("Layer "+obj.getLayerID());
		}
		
//		txt = child.getAttribute("acadcolor").getValue().toString();
//		obj.setAcadcolor(NewString.toInt(txt));
		
		txt = child.getAttribute("visible").getValue().toString();
		obj.setVisible(new Boolean(txt));
		
//		txt = child.getAttribute("locked").getValue().toString();
//		obj.set.setLocked(NewString.toBoolean(txt));
		
//		txt = child.getChild("property").getValue().toString();
//		obj.setProperty(txt);

		txt = child.getChild("framecolor").getAttribute("width").getValue().toString();
		obj.setLineWidth(new Double(txt));

		txt = child.getChild("framecolor").getValue().toString();
		obj.setLineColor(toColor(txt));
		obj.setTransparency(toTransparency(txt));

		txt = child.getChild("fillcolor").getValue().toString();
		obj.setFillColor(toColor(txt));
		obj.setTransparency(toTransparency(txt));

//		txt = child.getChild("pattern").getValue().toString();
//		obj.setPattern(txt);
		
		List listP = child.getChild("vector").getChildren("point");
		Iterator ip = listP.iterator();
		
		Element po = (Element) ip.next();
		txt = po.getValue().toString();
		Vertex p1=toVertex(txt);
		po = (Element) ip.next();
		txt = po.getValue().toString();
		Vertex p2=toVertex(txt);
		
// *** aus den beiden Vektoren wird die Richtung und die Lnge bestimmt ***
		if (p2.getX()-p1.getX()!=0 && p2.getY()-p1.getY()==0 && p2.getZ()-p1.getZ()==0) {
			obj.setExtrudingDirection(0);
			obj.setExtrusionStart(p1.getX());
			obj.setExtrusionStop(p2.getX());
		} else
		if (p2.getY()-p1.getY()!=0 && p2.getX()-p1.getX()==0 && p2.getZ()-p1.getZ()==0)  {
			obj.setExtrudingDirection(1);
			obj.setExtrusionStart(p1.getY());
			obj.setExtrusionStop(p2.getY());
		} else
		if (p2.getZ()-p1.getZ()!=0 && p2.getX()-p1.getX()==0 && p2.getY()-p1.getY()==0) {
			obj.setExtrudingDirection(2);
			obj.setExtrusionStart(p1.getZ());
			obj.setExtrusionStop(p2.getZ());
		} else {
			System.out.println("Fehler beim Lesen eines Zylinders: falsche Extrudierungsrichtung");
		}

// *** ende Umwandlung ***
		return obj;
	}

	public static void readSurface (ArrayList<CadObject> cadObjects, Element child, String path) throws Exception {

		// Creating a new CAD-object
	    URLClassLoader cl = new URLClassLoader(new URL[] { new File(path).toURL() });
		Class c = cl.loadClass("Surface");
		CadObject obj = (CadObject)c.newInstance();

		String txt = child.getAttribute("layerid").getValue().toString();
		obj.setLayerID(new Integer(txt));
		obj.setObjectId(currentID);
		currentID++;

		obj.setName("Surface");
		try {
			txt = child.getAttribute("filled").getValue().toString();
			obj.setFilled(new Boolean(txt));
		} catch (Exception e) {
			obj.setFilled(false);
		}

		try {
			txt = child.getAttribute("visible").getValue().toString();
			obj.setVisible(new Boolean(txt));
		} catch (Exception e) {
			obj.setVisible(true);
		}

//		obj.setVisible(true);  //wird wahrscheinlich noch durch das Layer-Attribut ersetzt
		
		List listP = child.getChild("triangleset").getChildren("triangle");
		Iterator ip = listP.iterator();
		Vector vertexList = new Vector<Vertex>();
		while (ip.hasNext()) {
//			CadObject surface = (CadObject)c.newInstance();
//			surface.setObjectId(currentID);
//			currentID++;
//			surface.setLayerID(obj.getLayerID());
//			surface.setName(obj.getName());
//			surface.setFilled(obj.isFilled());
//			surface.setVisible(obj.isVisible());
			Element po = (Element) ip.next();

			List listp2 = po.getChildren("point");
			Iterator ip2 = listp2.iterator();


			while (ip2.hasNext()) {
				Element po2 = (Element) ip2.next();
				txt = po2.getValue().toString();
				Vertex p1=toVertex(txt);
				vertexList.add(p1);

			}
		}
		obj.setVertexList(vertexList);
		cadObjects.add(obj);
	}

	public static void readCylinder (ArrayList<CadObject> cadObjects, Element child, String path) throws Exception {

		// Creating a new CAD-object
	    URLClassLoader cl = new URLClassLoader(new URL[] { new File(path).toURL() });
		Class c = cl.loadClass("Cylinder");
		CadObject obj = (CadObject)c.newInstance();

		String txt = child.getAttribute("layerid").getValue().toString();
		obj.setLayerID(new Integer(txt));

		txt = child.getAttribute("id").getValue().toString();
		obj.setObjectId(currentID);
		currentID++;

		obj.setName("Cylinder");
		try {
			txt = child.getAttribute("filled").getValue().toString();
			obj.setFilled(new Boolean(txt));
		} catch (Exception e) {
			obj.setFilled(false);
		}

		try {
			txt = child.getAttribute("visible").getValue().toString();
			obj.setVisible(new Boolean(txt));
		} catch (Exception e) {
			obj.setVisible(true);
		}

//		obj.setVisible(true);  //wird wahrscheinlich noch durch das Layer-Attribut ersetzt

		List listP = child.getChild("extrusion").getChild("vector").getChildren("point");
		Iterator ip = listP.iterator();
		Element po = (Element) ip.next();
		txt = po.getValue().toString();
		Vertex ext1=toVertex(txt);
		po = (Element) ip.next();
		txt = po.getValue().toString();
		Vertex ext2=toVertex(txt);

		if (ext2.getX()-ext1.getX()!=0 && ext2.getY()-ext1.getY()==0 && ext2.getZ()-ext1.getZ()==0) {
			obj.setExtrudingDirection(0);
		} else
		if (ext2.getY()-ext1.getY()!=0 && ext2.getX()-ext1.getX()==0 && ext2.getZ()-ext1.getZ()==0)  {
			obj.setExtrudingDirection(1);
		} else
		if (ext2.getZ()-ext1.getZ()!=0 && ext2.getX()-ext1.getX()==0 && ext2.getY()-ext1.getY()==0) {
			obj.setExtrudingDirection(2);
		} else {
			System.out.println("Fehler beim Lesen eines Zylinders: falsche Extrudierungsrichtung");
		}
		
		txt = child.getChild("radius").getAttribute("value").getValue().toString();
		float r=new Float(txt);
		txt = child.getChild("point").getValue().toString();
		Vertex center=toVertex(txt);
		Vector vertexList = new Vector<Vertex>();
//		float ht = (float) Math.sqrt(r*r);
		
		Vertex p1=null;
		Vertex p2=null;
		if (obj.getExtrudingDirection()==2) {
			p1=new Vertex(center.getX(),center.getY(),ext1.getZ());
			p2=new Vertex(center.getX()+r,center.getY()+r,ext2.getZ());
		} else
		if (obj.getExtrudingDirection()==1) {
			p1=new Vertex(center.getX(),ext1.getY(),center.getZ());
			p2=new Vertex(center.getX()+r,ext2.getY(),center.getZ()+r);
		} else
		if (obj.getExtrudingDirection()==0) {
			p1=new Vertex(ext1.getX(),center.getY(),center.getZ());
			p2=new Vertex(ext2.getX(),center.getY()+r,center.getZ()+r);
		}			
		vertexList.add(p1);
		vertexList.add(p2);
		obj.setVertexList(vertexList);
		cadObjects.add(obj);
		
	}
	
	public static void readSphere (ArrayList<CadObject> cadObjects, Element child, String path) throws Exception {

		// Creating a new CAD-object
	    URLClassLoader cl = new URLClassLoader(new URL[] { new File(path).toURL() });
		Class c = cl.loadClass("Sphere");
		CadObject obj = (CadObject)c.newInstance();

		String txt = child.getAttribute("layerid").getValue().toString();
		obj.setLayerID(new Integer(txt));

		txt = child.getAttribute("id").getValue().toString();
		obj.setObjectId(currentID);
		currentID++;

		obj.setName("Sphere");
		try {
			txt = child.getAttribute("filled").getValue().toString();
			obj.setFilled(new Boolean(txt));
		} catch (Exception e) {
			obj.setFilled(false);
		}

		try {
			txt = child.getAttribute("visible").getValue().toString();
			obj.setVisible(new Boolean(txt));
		} catch (Exception e) {
			obj.setVisible(true);
		}

//		obj.setVisible(true);  //wird wahrscheinlich noch durch das Layer-Attribut ersetzt

		txt = child.getChild("radius").getAttribute("value").getValue().toString();
		float r=new Float(txt);
		txt = child.getChild("point").getValue().toString();
		Vertex center=toVertex(txt);
		Vector vertexList = new Vector<Vertex>();
//		float ht = (float) Math.sqrt(r*r);
		Vertex p1=null;
		Vertex p2=null;
		p1=new Vertex(center.getX(),center.getY(),center.getZ());
		p2=new Vertex(center.getX()+r,center.getY()+r,center.getZ()+r);
		vertexList.add(p1);
		vertexList.add(p2);
		obj.setVertexList(vertexList);
		cadObjects.add(obj);
		
	}
	
	public static void readClosedPoly (ArrayList<CadObject> cadObjects, Vector<Layer> layers, Element child, String path) throws Exception {

		// Creating a new CAD-object
	    URLClassLoader cl = new URLClassLoader(new URL[] { new File(path).toURL() });
		Class c = cl.loadClass("ClosedPoly");
		CadObject obj = (CadObject)c.newInstance();

		String txt = child.getAttribute("layerid").getValue().toString();
		obj.setLayerID(new Integer(txt));

		txt = child.getAttribute("id").getValue().toString();
		obj.setObjectId(currentID);
		currentID++;
	
		List extP = child.getChild("extrusion").getChild("vector").getChildren("point");
		Iterator ip = extP.iterator();

		Element po1 = (Element) ip.next();
		txt = po1.getValue().toString();
		Vertex ext1=toVertex(txt);
		Element po2 = (Element) ip.next();
		txt = po2.getValue().toString();
		Vertex ext2=toVertex(txt);
		
		if (ext2.getX()-ext1.getX()!=0 && ext2.getY()-ext1.getY()==0 && ext2.getZ()-ext1.getZ()==0) {
			obj.setExtrudingDirection(0);
			obj.setExtrusionStart(ext1.getX());
			obj.setExtrusionStop(ext2.getX());
		} else
		if (ext2.getY()-ext1.getY()!=0 && ext2.getX()-ext1.getX()==0 && ext2.getZ()-ext1.getZ()==0)  {
			obj.setExtrudingDirection(1);
			obj.setExtrusionStart(ext1.getY());
			obj.setExtrusionStop(ext2.getY());
		} else
		if (ext2.getZ()-ext1.getZ()!=0 && ext2.getX()-ext1.getX()==0 && ext2.getY()-ext1.getY()==0) {
			obj.setExtrudingDirection(2);
			obj.setExtrusionStart(ext1.getZ());
			obj.setExtrusionStop(ext2.getZ());
		} else {
			System.out.println("Fehler beim Lesen eines ClosedPoly: falsche Extrudierungsrichtung");
		}

		
//		obj.setExtrudingDirection(layers.get(obj.getLayerID()).getExtrudingDirection());
//		obj.setExtrusionStart(layers.get(obj.getLayerID()).getExtrusionStart());
//		obj.setExtrusionStop(layers.get(obj.getLayerID()).getExtrusionStop());

		obj.setName("ClosedPoly");
		try {
			txt = child.getAttribute("filled").getValue().toString();
			obj.setFilled(new Boolean(txt));
		} catch (Exception e) {
			obj.setFilled(false);
		}

		try {
			txt = child.getAttribute("visible").getValue().toString();
			obj.setVisible(new Boolean(txt));
		} catch (Exception e) {
			obj.setVisible(true);
		}

		obj.setVisible(true);  //wird wahrscheinlich noch durch das Layer-Attribut ersetzt

		txt = child.getChild("pointset").getAttribute("qty").getValue().toString();

		Vector vertexList = new Vector<Vertex>();

		int i=0;
		List listP = child.getChild("pointset").getChildren("point");
		ip = listP.iterator();
		while (ip.hasNext()) {
			Element po = (Element) ip.next();
			txt = po.getValue().toString();
			Vertex p1=toVertex(txt);
			vertexList.add(p1);
			i++;
		}
		obj.setNumberOfPoints(i);
		obj.setVertexList(vertexList);
	    cadObjects.add(obj);	
//		return obj;
	}

	public static void readLinPoly (ArrayList<CadObject> cadObjects, Element child, String path) throws Exception {

		// Creating a new CAD-object
	    URLClassLoader cl = new URLClassLoader(new URL[] { new File(path).toURL() });
		Class c = cl.loadClass("PolyLineNPoints");
		CadObject obj = (CadObject)c.newInstance();

		String txt = child.getAttribute("layerid").getValue().toString();
		obj.setLayerID(new Integer(txt));

		txt = child.getAttribute("id").getValue().toString();
		obj.setObjectId(currentID);
		currentID++;

		obj.setName("LinPoly");
		try {
			txt = child.getAttribute("filled").getValue().toString();
			obj.setFilled(new Boolean(txt));
		} catch (Exception e) {
			obj.setFilled(false);
		}

		try {
			txt = child.getAttribute("visible").getValue().toString();
			obj.setVisible(new Boolean(txt));
		} catch (Exception e) {
			obj.setVisible(true);
		}

//		obj.setVisible(true);  //wird wahrscheinlich noch durch das Layer-Attribut ersetzt

//		txt = child.getChild("pointset").getAttribute("qty").getValue().toString();

		Vector vertexList = new Vector<Vertex>();

		int i=0;
		List listP = child.getChildren("point");
		Iterator ip = listP.iterator();
		while (ip.hasNext()) {
			Element po = (Element) ip.next();
			txt = po.getValue().toString();
			Vertex p1=toVertex(txt);
			vertexList.add(p1);
			i++;
		}
		obj.setNumberOfPoints(i);
		obj.setVertexList(vertexList);
	    cadObjects.add(obj);	
//		return obj;
	}

	
	public static void readBox (ArrayList<CadObject> cadObjects, Element child, String path) throws Exception {

		// Creating a new CAD-object
	    URLClassLoader cl = new URLClassLoader(new URL[] { new File(path).toURL() });
		Class c = cl.loadClass("SimpleRectangle");
		CadObject obj = (CadObject)c.newInstance();

		String txt = child.getAttribute("layerid").getValue().toString();
		obj.setLayerID(new Integer(txt));

		txt = child.getAttribute("id").getValue().toString();
		obj.setName("Quader");

		try {
			txt = child.getAttribute("filled").getValue().toString();
			obj.setFilled(new Boolean(txt));
		} catch (Exception e) {
			obj.setFilled(false);
		}

		try {
			txt = child.getAttribute("visible").getValue().toString();
			obj.setVisible(new Boolean(txt));
		} catch (Exception e) {
			obj.setVisible(true);
		}

		List quaders = child.getChild("quaderset").getChildren("quader");
		Iterator quaderIp = quaders.iterator();
		
		while (quaderIp.hasNext()) {
			
			CadObject box = (CadObject)c.newInstance();
			box.setObjectId(currentID);
			currentID++;
			box.setLayerID(obj.getLayerID());
			box.setName(obj.getName());
			box.setFilled(obj.isFilled());
			box.setVisible(obj.isVisible());

			Element quader = (Element)quaderIp.next();
		
			List listP = quader.getChildren("point");
			Iterator ip = listP.iterator();
	
			Vector vertexList = new Vector<Vertex>();
		
			Element po = (Element) ip.next();
			txt = po.getValue().toString();
			Vertex p1=toVertex(txt);
			vertexList.add(p1);
	
			po = (Element) ip.next();
			txt = po.getValue().toString();
			Vertex p2=toVertex(txt);
			vertexList.add(p2);
			
			box.setVertexList(vertexList);
			cadObjects.add(box);
		}
//		return obj;
	}

	
	
	
	private String selectFile(String path)
	{
		JFileChooser fc = new JFileChooser();
		fc.setCurrentDirectory(new File("c:/Developer/XML Files/XML Layer Files/"));
//		fc.setCurrentDirectory(new File(path));
		File file = null;
		String fileName = null;
		
		int retVal = fc.showOpenDialog( null );

		if(retVal==JFileChooser.APPROVE_OPTION ) {
			file = fc.getSelectedFile();
			fileName = file.getAbsolutePath();
			currentFilePath = file.getPath();
		}
		else
			System.out.println( "Auswahl abgebrochen" );	
		
		return fileName;
	}

	public static Color toColor(String str) {
		Color vec=null;
		Boolean rgb = false;
		StringTokenizer tok = new StringTokenizer(str,",");
		String x = tok.nextToken();
		String y = tok.nextToken();
		String z = tok.nextToken();
		System.out.println(x+y+z);
		Float xx = new Float(x);
		Float yy = new Float(y);
		Float zz = new Float(z);
		if (xx>1f || yy>1f || zz>1f) rgb=true;
		if (!rgb) {
			vec = new Color(new Float(x),new Float(y),new Float(z));
		} else {
			vec = new Color(new Integer(x), new Integer(y), new Integer(z));
		}
		return vec;
	}
	public static float toTransparency(String str) {
		StringTokenizer tok = new StringTokenizer(str,",");
		String x = tok.nextToken();
		String y = tok.nextToken();
		String z = tok.nextToken();
		String t = tok.nextToken();
		return new Float(t);
	}
	public static Vertex toVertex(String str) {
		StringTokenizer tok = new StringTokenizer(str,",");
		String x = tok.nextToken();
		String y = tok.nextToken();
		String z = tok.nextToken();
		return new Vertex(new Double(x),new Double(y),new Double(z));
//		return vec;
	}


}
