/*
Class-name: SimpleEllipse.java. Object-Plugin for the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a (soon) integrated plugable simulation-interface

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

import java.awt.Shape;
import java.awt.geom.Arc2D;
import java.awt.geom.Rectangle2D;

/**
 * SimpleLine: Paints a simple ellipse from point x1,y1 to x2,y2
 * @author Karsten Bettray
 */
public class SimpleCylinder extends EmptyCadObject // implements FemObject
{
	public SimpleCylinder()
	{
		setName("SimpleCylinder");
		setGeometryTyp(EmptyCadObject.GEOMETRY_3D_CYLINDER);
		setFillable(true);
	}
	public SimpleCylinder(String name)
	{
		setName(name);
		setGeometryTyp(EmptyCadObject.GEOMETRY_3D_CYLINDER);
		setFillable(true);
	}

	/**
	 * @param panelID Parameter is [0-2]. Shows, which panel-view is activ. Needed for the front-, top- or side-view. 
	 */
	public Shape initShape(int panelID, double scale)
	{	
		double x1, y1, z1, x2, y2, z2;
		
		x1 = (vertexList.get(0).getX() * scale); y1 = (vertexList.get(0).getY() * scale); z1 = (vertexList.get(0).getZ() * scale);
		x2 = (vertexList.get(1).getX() * scale); y2 = (vertexList.get(1).getY() * scale); z2 = (vertexList.get(1).getZ() * scale);

		shapeXY = generateShape( x1, y1, z1, x2, y2, z2, 0);
		shapeZY = generateShape( x1, y1, z1, x2, y2, z2, 1);
		shapeXZ = generateShape( x1, y1, z1, x2, y2, z2, 2);

		if(panelID==0) shape = shapeXY;
		else if(panelID==1) shape = shapeZY;
		else shape = shapeXZ;

		return shape;
	}
	
	private Shape generateShape(double x1, double y1, double z1, double x2, double y2, double z2, int panelID)
	{
		Shape shape;
		double radius, dx, dy, dz, doubleRadius;
		
		// Calculating the differences 
		dx = x2 - x1;
		dy = y2 - y1;
		dz = z2 - z1;
		
		// calculating the radius and the height
		if(builtInView==0)
			radius = (float)Math.sqrt(dx*dx + dy*dy);
		else if(builtInView==1)
			radius = (float)Math.sqrt(dy*dy + dz*dz);
		else
			radius = (float)Math.sqrt(dx*dx + dz*dz);
		
//		System.out.println("Math.abs(extrusionStop - extrusionStart): "+Math.abs(extrusionStop - extrusionStart));
		
		if(panelID==builtInView)
			shape = getArc2D(x1 - radius, y1 - radius, z1 - radius, radius*2d, radius*2d, panelID);
		else
			shape = getRechtange2D(x1, y1, z1, radius, Math.abs(extrusionStop - extrusionStart), panelID);
			
		return shape;
	}

	
	private Shape getArc2D(double x1, double y1, double z1, double width, double height, int panelID)
	{
		if(panelID==0)
			return new Arc2D.Double(x1, y1, width, height, 0, 360, Arc2D.CHORD);
		else if(panelID==1)
			return new Arc2D.Double(z1, y1, width, height, 0, 360, Arc2D.CHORD);
		else
			return new Arc2D.Double(x1, z1, width, height, 0, 360, Arc2D.CHORD);
	}
	
	private Shape getRechtange2D(double x, double y, double z, double radius, double height, int panelID)
	{
		double doubleRadius = 2*radius;
		
		if(builtInView==0) {
			if(panelID==1)
				return new Rectangle2D.Double(z, y - radius, height, doubleRadius);
			else if(panelID==2)
				return new Rectangle2D.Double(x - radius, z, doubleRadius, height);
		}
		else if(builtInView==1) {
			if(panelID==0)
				return new Rectangle2D.Double(x, y - radius, height, doubleRadius);
			else if(panelID==2)
				return new Rectangle2D.Double(x, z - radius, height, doubleRadius);
		}
		else {
			if(panelID==0)
				return new Rectangle2D.Double(x - radius, y, doubleRadius, height);
			else if(panelID==1)
				return new Rectangle2D.Double(z - radius, y, doubleRadius, height);
		}
		
		return null;
	}
	
	public int getLineTyp()
	{
		return EmptyCadObject.GEOMETRY_3D_CYLINDER;
	}
	
	public void finalizeObject(int quadrant)
	{
//		double x1, y1, z1, x2, y2, z2;
//
//		x1 = (vertexList.get(0).getX()); y1 = (vertexList.get(0).getY()); z1 = (vertexList.get(0).getZ());
//		x2 = (vertexList.get(1).getX()); y2 = (vertexList.get(1).getY()); z2 = (vertexList.get(1).getZ());
//		
		

	}

}
