/*
Class-name: SimpleEllipse.java. Object-Plugin for the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a (soon) integrated plugable simulation-interface

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

import java.awt.Shape;
import java.awt.geom.Arc2D;
import java.awt.geom.Rectangle2D;

/**
 * SimpleLine: Paints a simple ellipse from point x1,y1 to x2,y2
 * @author Karsten Bettray
 */
public class SimpleEllipse extends EmptyCadObject // implements FemObject
{
	public SimpleEllipse()
	{
		setName("SimpleCircle");
		setGeometryTyp(EmptyCadObject.GEOMETRY_2D_ELLIPSE);
		setFillable(true);
	}
	public SimpleEllipse(String name)
	{
		setName(name);
		setGeometryTyp(EmptyCadObject.GEOMETRY_2D_ELLIPSE);
		setFillable(true);
	}

	/**
	 * @param panelID Parameter is [0-2]. Shows, which panel-view is activ. Needed for the front-, top- or side-view. 
	 */
	public Shape initShape(int panelID, double scale)
	{	
		double x1, y1, x2, y2;
		
		x1 = (vertexList.get(0).getX() * scale); y1 = (vertexList.get(0).getY() * scale);
		x2 = (vertexList.get(1).getX() * scale); y2 = (vertexList.get(1).getY() * scale);
		shapeXY = generateShape( x1, y1, x2, y2);

		x1 = (vertexList.get(0).getZ() * scale); y1 = (vertexList.get(0).getY() * scale);
		x2 = (vertexList.get(1).getZ() * scale); y2 = (vertexList.get(1).getY() * scale);
		shapeZY = generateShape( x1, y1, x2, y2);

		x1 = (vertexList.get(0).getX() * scale); y1 = (vertexList.get(0).getZ() * scale);
		x2 = (vertexList.get(1).getX() * scale); y2 = (vertexList.get(1).getZ() * scale);
		shapeXZ = generateShape( x1, y1, x2, y2);

		if(panelID==0) shape = shapeXY;
		else if(panelID==1) shape = shapeZY;
		else shape = shapeXZ;

		return shape;
	}
	
	private Shape generateShape(double x1, double y1, double x2, double y2)
	{
		Shape shape;
		
		if(x1 <= x2 && y1 <= y2) {
			shape = getArc2D(2*x1 - x2, 2*y1 - y2, (x2 - x1)*2d, (y2 - y1)*2d);
		}
		else if(x1 > x2 && y1 <= y2) {
			shape = getArc2D(x2, 2*y1 - y2, (x1 - x2)*2d, (y2 - y1)*2d);
		}
		else if(x1 <= x2 && y1 > y2) {
			shape = getArc2D(2*x1 - x2, y2, (x2 - x1)*2d, (y1 - y2)*2d);
		}
		else { // x1 > x2 && y1 > y2
			shape = getArc2D(x2, y2, (x1 - x2)*2d, (y1 - y2)*2d);
		}

		return shape;
	}

	
	private Shape getArc2D(double x1, double y1, double x2, double y2)
	{
		return new Arc2D.Double(x1, y1, x2, y2, 0, 360, Arc2D.CHORD);
	}
	
	public int getLineTyp() {
		return EmptyCadObject.GEOMETRY_2D_ELLIPSE;
	}
}
