/*
Class-name: SimpleLine.java. Object-Plugin for the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a (soon) integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

import interfaces.CadObject;

import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;

/**
 * SimpleLine: Paints a simple line from point x1,y1 to x2,y2
 * @author Karsten Bettray
 */
public class SimpleLine extends EmptyCadObject implements CadObject
{
	public SimpleLine()
	{
		setName("SimpleLine");
		setNumberOfPoints(2);
		setGeometryTyp(EmptyCadObject.GEOMETRY_2D_LINE);
		setFillable(false);
		initEnvironment();
	}
	public SimpleLine(String name)
	{
		setName(name);
		setNumberOfPoints(2);
		setGeometryTyp(EmptyCadObject.GEOMETRY_2D_LINE);
		setFillable(false);
		initEnvironment();
	}
	
	/**
	 * @param panelID Parameter is [0-2]. Shows, which panel-view is activ. Needed for the front-, top- or side-view.
	 * 
	 */
	public Shape initShape(int panelID, double scale)
	{
		double x1, y1, z1, x2, y2, z2;
		x1 = (vertexList.get(0).getX() * scale); y1 = (vertexList.get(0).getY() * scale); z1 = (vertexList.get(0).getZ() * scale);
		x2 = (vertexList.get(1).getX() * scale); y2 = (vertexList.get(1).getY() * scale); z2 = (vertexList.get(1).getZ() * scale);

		shapeXY = new Line2D.Double((float)x1, (float)y1, (float)x2, (float)y2);
		shapeZY = new Line2D.Double((float)z1, (float)y1, (float)z2, (float)y2);
		shapeXZ = new Line2D.Double((float)x1, (float)z1, (float)x2, (float)z2);

		if(panelID==0)
			shape = shapeXY;
		else if(panelID==1)
			shape = shapeZY;
		else
			shape = shapeXZ;
		
		return shape;
		
		
	}
}
