/*
Class-name: Cylinder.java. Object-Plugin for the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a integrated plugable simulation-interface 

Copyright (C) 2006 Kay Dnzer	

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

import java.awt.Shape;
import java.awt.geom.Arc2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;


public class Surface extends EmptyCadObject {

	GeneralPath gp = null;

	public Surface()
	{
		setName("Surface");
		setNumberOfPoints(3);
		setGeometryTyp(EmptyCadObject.GEOMETRY_2D_POLYGON);
		setFillable(true);
		initEnvironment();
	}

	public Surface(String name)
	{
		setName(name);
		setNumberOfPoints(3);
		setGeometryTyp(EmptyCadObject.GEOMETRY_2D_POLYGON);
		setFillable(true);
		initEnvironment();
	}

	public Shape getShape(int panelID, double scale)
	{	
		double x1, y1, x2, y2, x3, y3;
		gp = new GeneralPath();
		
		if(panelID==0) {
			for(int i=0; i<vertexList.size(); i=i+3) {
				gp.moveTo((float)(vertexList.get(i).getX() * scale), (float)(vertexList.get(i).getY() * scale));
				gp.lineTo((float)(vertexList.get(i+1).getX() * scale), (float)(vertexList.get(i+1).getY() * scale));
				gp.lineTo((float)(vertexList.get(i+2).getX() * scale), (float)(vertexList.get(i+2).getY() * scale));
				gp.closePath();
//				gp.lineTo((float)(vertexList.get(i).getX() * scale), (float)(vertexList.get(i).getY() * scale));
			}
		} 
		else if(panelID==1) {
			for(int i=0; i<vertexList.size(); i=i+3) {
				gp.moveTo((float)(vertexList.get(i).getZ() * scale), (float)(vertexList.get(i).getY() * scale));
				gp.lineTo((float)(vertexList.get(i+1).getZ() * scale), (float)(vertexList.get(i+1).getY() * scale));
				gp.lineTo((float)(vertexList.get(i+2).getZ() * scale), (float)(vertexList.get(i+2).getY() * scale));
				gp.closePath();
//				gp.lineTo((float)(vertexList.get(i).getZ() * scale), (float)(vertexList.get(i).getY() * scale));
			}
		} 
		else {
			for(int i=0; i<vertexList.size(); i=i+3) {
				gp.moveTo((float)(vertexList.get(i).getX() * scale), (float)(vertexList.get(i).getZ() * scale));
				gp.lineTo((float)(vertexList.get(i+1).getX() * scale), (float)(vertexList.get(i+1).getZ() * scale));
				gp.lineTo((float)(vertexList.get(i+2).getX() * scale), (float)(vertexList.get(i+2).getZ() * scale));
				gp.closePath();
//				gp.lineTo((float)(vertexList.get(i).getX() * scale), (float)(vertexList.get(i).getZ() * scale));
			}
		}
		
		shape = gp;

		return shape;
	}
	
	
	public int getLineTyp() {
		return EmptyCadObject.GEOMETRY_2D_POLYGON;
	}

}
