/*
Class-name: WriteXML.java. Object-Plugin for the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a integrated plugable simulation-interface 

Copyright (C) 2006 Kay Dnzer	

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

import interfaces.CadObject;
import interfaces.Layer;

import java.io.*;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JFileChooser;

import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;


public class WriteXML extends EmptyExportImportObject {

	private String currentFilePath = null;

	public void exportObjects(ArrayList<CadObject> cadObjects, Vector<Layer> layers, String path)
	{
		CadObject swapObject;

		String fileName = selectFile(path);

		org.jdom.Document d = new org.jdom.Document();

		Element root = new Element("CADFile");
		root.setAttribute("Version","1.1");

//		 Layerset schreiben:
		Element layerset = new Element("layerset");
		int anzlayer=0;
		for(int i=0; i<layers.size(); i++) {
			swapObject = layers.get(i);
			if (!swapObject.getName().equals("Default-Layer")) {
				layerset.addContent(writeLayer(swapObject));
				anzlayer++;
			}
		}
		layerset.setAttribute("qty",String.valueOf(anzlayer));
		root.addContent(layerset);
		
//		 Objects schreiben:
		Element polyset = new Element("polyset");
		polyset.setAttribute("id","0");
		Element boxset = new Element("boxset");
		boxset.setAttribute("id","0");
		Element surfaceset = new Element("surfaceset");
		surfaceset.setAttribute("id","0");
		int anzpoly=0;
		int anzbox=0;
		int anzsurface=0;
		for(int i=0; i<cadObjects.size(); i++) {
			swapObject = cadObjects.get(i);
			
			if (swapObject.getName().equals("SimpleCircle")) {
				polyset.addContent(writeSimpleCircle(swapObject));
				anzpoly++;
			} else
			if (swapObject.getName().equals("SimpleRectangle")) {
				boxset.addContent(writeSimpleRectangle(swapObject));
				anzbox++;
			} else
			if (swapObject.getName().equals("SimpleLine")||swapObject.getName().equals("PolyLine3Points")||swapObject.getName().equals("PolyLine4Points")) {
				polyset.addContent(writePolyLine(swapObject));
				anzpoly++;
			} else
			if (swapObject.getName().equals("Surface")) {
				surfaceset.addContent(writeSurface(swapObject));
				anzsurface++;
			} else
			if (swapObject.getName().equals("Sphere")) {
				polyset.addContent(writeSphere(swapObject));
				anzpoly++;
			} else
			if (swapObject.getName().equals("Cylinder")) {
				polyset.addContent(writeCylinder(swapObject));
				anzpoly++;
			} else
			if (swapObject.getName().equals("ClosedPoly")) {
				polyset.addContent(writeClosedPoly(swapObject));
				anzpoly++;
			} else
			if (swapObject.getName().equals("Quader")) {
				boxset.addContent(writeBox(swapObject));
				anzbox++;
			} 
		}
		polyset.setAttribute("qty",String.valueOf(anzpoly));
		boxset.setAttribute("qty",String.valueOf(anzbox));
		surfaceset.setAttribute("qty",String.valueOf(anzsurface));
		if (anzbox>0) root.addContent(boxset);
		if (anzpoly>0) root.addContent(polyset);
		if (anzsurface>0) root.addContent(surfaceset);
		
		d.setRootElement(root);
		try {
			FileOutputStream export = new FileOutputStream(fileName);
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			out.output(d, export);
			export.flush();
			export.close();
		} catch (IOException e) {}
	}

	private static Element writeBox (CadObject obj) {
		Element child = new Element("box");
		child.setAttribute("id",String.valueOf(obj.getObjectId()));
		child.setAttribute("classname",obj.getClass().getCanonicalName());
		child.setAttribute("name",obj.getName());
		child.setAttribute("layerid",String.valueOf(obj.getLayerID()));
		child.setAttribute("filled",String.valueOf(obj.isFilled()));
		child.setAttribute("visible",String.valueOf(obj.isVisible()));
		

		Element set = new Element("quaderset");
		set.setAttribute("id",String.valueOf(obj.getObjectId()));
		set.setAttribute("qty","1");
		Element box = new Element("quader");
		box.setAttribute("nr","0");
		box.setAttribute("id","0");
		Element point = new Element("point");
		point.setAttribute("nr","0");
		point.addContent(obj.getVertexList().get(0).getX()+","+obj.getVertexList().get(0).getY()+","+obj.getVertexList().get(0).getZ());
		box.addContent(point);
		point = new Element("point");
		point.setAttribute("nr","1");
		point.addContent(obj.getVertexList().get(1).getX()+","+obj.getVertexList().get(1).getY()+","+obj.getVertexList().get(1).getZ());
		box.addContent(point);
		set.addContent(box);
		child.addContent(set);
		return child;
	}


	private static Element writeClosedPoly (CadObject obj) {
		Element child = new Element("poly");
		child.setAttribute("id",String.valueOf(obj.getObjectId()));
		child.setAttribute("classname",obj.getClass().getCanonicalName());
		child.setAttribute("name",obj.getName());
		child.setAttribute("layerid",String.valueOf(obj.getLayerID()));
		child.setAttribute("filled",String.valueOf(obj.isFilled()));
		child.setAttribute("visible",String.valueOf(obj.isVisible()));
		child.setAttribute("structure","extruded");

		Element pointset = new Element("pointset");
		pointset.setAttribute("id",String.valueOf(obj.getObjectId()));
		int anz = obj.getVertexList().size();
		pointset.setAttribute("qty",String.valueOf(anz));
		
		for (int i=0; i<anz; i++) {
			Element point = new Element("point");
			point.setAttribute("nr",String.valueOf(i));
			point.addContent(obj.getVertexList().get(i).getX()+","+obj.getVertexList().get(i).getY()+","+obj.getVertexList().get(i).getZ());
			pointset.addContent(point);
		}
		child.addContent(pointset);

		
		Element extrusion = new Element("extrusion");
		Element vector = new Element("vector");
		vector.setAttribute("nr","0");
		vector.setAttribute("id","1");
		Element point = new Element("point");
		point.setAttribute("nr",String.valueOf(anz));
		point.setAttribute("type","start");
		float start = (float)obj.getExtrusionStart();
		if (obj.getExtrudingDirection()==0) {
			point.addContent(start+",0.0,0.0");
		} else
		if (obj.getExtrudingDirection()==1) {
			point.addContent("0.0,"+start+",0.0");
		} else
		if (obj.getExtrudingDirection()==2) {
			point.addContent("0.0,0.0,"+start);
		}
		vector.addContent(point);
		point = new Element("point");
		point.setAttribute("nr",String.valueOf(anz+1));
		point.setAttribute("type","end");
		float stop = (float)obj.getExtrusionStop();
		if (obj.getExtrudingDirection()==0) {
			point.addContent(stop+",0.0,0.0");
		} else
		if (obj.getExtrudingDirection()==1) {
			point.addContent("0.0,"+stop+",0.0");
		} else
		if (obj.getExtrudingDirection()==2) {
			point.addContent("0.0,0.0,"+stop);
		}
		vector.addContent(point);
		extrusion.addContent(vector);
		child.addContent(extrusion);

		return child;
	}	


	private static Element writeCylinder (CadObject obj) {
		Element child = new Element("poly");
		child.setAttribute("id",String.valueOf(obj.getObjectId()));
		child.setAttribute("classname",obj.getClass().getCanonicalName());
		child.setAttribute("name",obj.getName());
		child.setAttribute("layerid",String.valueOf(obj.getLayerID()));
		child.setAttribute("filled",String.valueOf(obj.isFilled()));
		child.setAttribute("visible",String.valueOf(obj.isVisible()));
		child.setAttribute("structure","extruded");

		Element radius = new Element("radius");
		float r=(float)obj.getVertexList().get(1).getX()-(float)obj.getVertexList().get(0).getX();
		radius.setAttribute("value",String.valueOf(r));
		child.addContent(radius);
		
		Element center = new Element("point");
		center.setAttribute("nr",String.valueOf(0));
		center.addContent(obj.getVertexList().get(0).getX()+","+obj.getVertexList().get(0).getY()+","+obj.getVertexList().get(0).getZ());
		child.addContent(center);
		
		Element extrusion = new Element("extrusion");
		Element vector = new Element("vector");
		vector.setAttribute("nr","0");
		vector.setAttribute("id","1");
		Element point = new Element("point");
		point.setAttribute("nr","1");
		point.setAttribute("type","start");
		if (obj.getExtrudingDirection()==0) {
			float start = (float)obj.getVertexList().get(0).getX();
			point.addContent(start+",0.0,0.0");
		} else
		if (obj.getExtrudingDirection()==1) {
			float start = (float)obj.getVertexList().get(0).getY();
			point.addContent("0.0,"+start+",0.0");
		} else
		if (obj.getExtrudingDirection()==2) {
			float start = (float)obj.getVertexList().get(0).getZ();
			point.addContent("0.0,0.0,"+start);
		}
		vector.addContent(point);
		point = new Element("point");
		point.setAttribute("nr","2");
		point.setAttribute("type","end");
		if (obj.getExtrudingDirection()==0) {
			float stop = (float)obj.getVertexList().get(1).getX();
			point.addContent(stop+",0.0,0.0");
		} else
		if (obj.getExtrudingDirection()==1) {
			float stop = (float)obj.getVertexList().get(1).getY();
			point.addContent("0.0,"+stop+",0.0");
		} else
		if (obj.getExtrudingDirection()==2) {
			float stop = (float)obj.getVertexList().get(1).getZ();
			point.addContent("0.0,0.0,"+stop);
		}
		vector.addContent(point);
		extrusion.addContent(vector);
		child.addContent(extrusion);

		return child;
	}	

	private static Element writeSphere (CadObject obj) {
		Element child = new Element("poly");
		child.setAttribute("id",String.valueOf(obj.getObjectId()));
		child.setAttribute("classname",obj.getClass().getCanonicalName());
		child.setAttribute("name",obj.getName());
		child.setAttribute("layerid",String.valueOf(obj.getLayerID()));
		child.setAttribute("filled",String.valueOf(obj.isFilled()));
		child.setAttribute("visible",String.valueOf(obj.isVisible()));
		child.setAttribute("structure","rotated");

		Element radius = new Element("radius");
		float r=(float)obj.getVertexList().get(1).getX()-(float)obj.getVertexList().get(0).getX();
		radius.setAttribute("value",String.valueOf(r));
		child.addContent(radius);
		
		Element center = new Element("point");
		center.setAttribute("nr",String.valueOf(0));
		center.addContent(obj.getVertexList().get(0).getX()+","+obj.getVertexList().get(0).getY()+","+obj.getVertexList().get(0).getZ());
		child.addContent(center);

		return child;
	}	
		

	private static Element writeSurface (CadObject obj) {
		Element child = new Element("surface");
		child.setAttribute("id",String.valueOf(obj.getObjectId()));
		child.setAttribute("classname",obj.getClass().getCanonicalName());
		child.setAttribute("name",obj.getName());
		child.setAttribute("layerid",String.valueOf(obj.getLayerID()));
		child.setAttribute("filled",String.valueOf(obj.isFilled()));
		child.setAttribute("visible",String.valueOf(obj.isVisible()));

		Element set = new Element("triangleset");
		set.setAttribute("id",String.valueOf(obj.getObjectId()));

		int anz=0;
		for (int i=0; i<obj.getVertexList().size(); i+=3) {
			Element triangle = new Element("triangle");
			triangle.setAttribute("id",String.valueOf(0));
			triangle.setAttribute("nr",String.valueOf(anz));
			
			for (int j=0; j<=2; j++) {
				Element point = new Element("point");
				point.setAttribute("nr",String.valueOf(i+j));
				point.addContent(obj.getVertexList().get(i+j).getX()+","+obj.getVertexList().get(i+j).getY()+","+obj.getVertexList().get(i+j).getZ());
				triangle.addContent(point);
			}
			set.addContent(triangle);
			anz++;
		}
		child.addContent(set);
		return child;
	}	
		
	private static Element writeSimpleRectangle (CadObject obj) {
		Element child = new Element("box");
		child.setAttribute("id",String.valueOf(obj.getObjectId()));
		child.setAttribute("classname",obj.getClass().getCanonicalName());
		child.setAttribute("name",obj.getName());
		child.setAttribute("layerid",String.valueOf(obj.getLayerID()));
		child.setAttribute("filled",String.valueOf(obj.isFilled()));
		child.setAttribute("visible",String.valueOf(obj.isVisible()));
		

		Element set = new Element("quaderset");
		set.setAttribute("id",String.valueOf(obj.getObjectId()));
		set.setAttribute("qty","1");
		Element box = new Element("quader");
		set.setAttribute("nr","0");
		set.setAttribute("id","0");
		Element point = new Element("point");
		point.setAttribute("nr","0");
		point.addContent(obj.getVertexList().get(0).getX()+","+obj.getVertexList().get(0).getY()+","+obj.getVertexList().get(0).getZ());
		box.addContent(point);
		point = new Element("point");
		point.setAttribute("nr","1");
		point.addContent(obj.getVertexList().get(1).getX()+","+obj.getVertexList().get(1).getY()+","+obj.getVertexList().get(1).getZ());
		box.addContent(point);
		set.addContent(box);
		child.addContent(set);
		return child;
	}

	private static Element writeSimpleCircle (CadObject obj) {
		Element child = new Element("poly");
		child.setAttribute("id",String.valueOf(obj.getObjectId()));
		child.setAttribute("name",obj.getName());
		child.setAttribute("layerid",String.valueOf(obj.getLayerID()));
		
		Element point = new Element("point");
		point.setAttribute("nr","0");
		point.addContent(obj.getVertexList().get(0).getX()+","+obj.getVertexList().get(0).getY()+","+obj.getVertexList().get(0).getZ());
		child.addContent(point);

		point = new Element("point");
		point.setAttribute("nr","1");
		point.addContent(obj.getVertexList().get(1).getX()+","+obj.getVertexList().get(1).getY()+","+obj.getVertexList().get(1).getZ());
		child.addContent(point);
		return child;
	} 

	private static Element writePolyLine (CadObject obj) {
		Element child = new Element("poly");
		child.setAttribute("id",String.valueOf(obj.getObjectId()));
		child.setAttribute("name",obj.getName());
		child.setAttribute("layerid",String.valueOf(obj.getLayerID()));
		child.setAttribute("structure","extruded");
		
		Element pointset = new Element("pointset");
		pointset.setAttribute("id",String.valueOf(obj.getObjectId()));
		pointset.setAttribute("qty",String.valueOf(obj.getVertexList().size()));
		
		for (int i=0; i<obj.getVertexList().size(); i++) {
			Element point = new Element("point");
			point.setAttribute("nr",String.valueOf(i));
			point.addContent(obj.getVertexList().get(i).getX()+","+obj.getVertexList().get(i).getY()+","+obj.getVertexList().get(i).getZ());
			pointset.addContent(point);
		}
		child.addContent(pointset);
		return child;
	} 

	private static Element writeLayer (CadObject layer) {
		Element child = new Element("layer");
		child.setAttribute("id",String.valueOf(layer.getLayerID()));
		child.setAttribute("name",layer.getName());
		child.setAttribute("acadcolor","0");
		child.setAttribute("visible",String.valueOf(layer.isVisible()));
		child.setAttribute("locked","false");
		
		Element prop = new Element("property");
		child.addContent(prop);
		
		Element framecol = new Element("framecolor");
		framecol.setAttribute("width",String.valueOf(layer.getLineWidth()));
		framecol.setAttribute("type","rgba");
		framecol.addContent(layer.getLineColor().getRed()+","+layer.getLineColor().getGreen()+","+layer.getLineColor().getBlue()+","+layer.getTransparency());
		child.addContent(framecol);
		
		Element fillcol = new Element("fillcolor");
		fillcol.setAttribute("type","rgba");
		fillcol.addContent(layer.getFillColor().getRed()+","+layer.getFillColor().getGreen()+","+layer.getFillColor().getBlue()+","+layer.getTransparency());
		child.addContent(fillcol);
		
		Element pattern = new Element("pattern");
		pattern.addContent("No Fill");
		child.addContent(pattern);
		
		Element vector = new Element("vector");
		vector.setAttribute("nr","0");
		vector.setAttribute("id","1");
		Element point = new Element("point");
		point.setAttribute("nr","0");
		point.setAttribute("type","start");
		if (layer.getExtrudingDirection()==0) {
			point.addContent(layer.getExtrusionStart()+",0.0,0.0");
		} else
		if (layer.getExtrudingDirection()==1) {
			point.addContent("0.0,"+layer.getExtrusionStart()+",0.0");
		} else
		if (layer.getExtrudingDirection()==2) {
			point.addContent("0.0,0.0,"+layer.getExtrusionStart());
		}
		vector.addContent(point);
		point = new Element("point");
		point.setAttribute("nr","1");
		point.setAttribute("type","stop");
		if (layer.getExtrudingDirection()==0) {
			point.addContent(layer.getExtrusionStop()+",0.0,0.0");
		} else
		if (layer.getExtrudingDirection()==1) {
			point.addContent("0.0,"+layer.getExtrusionStop()+",0.0");
		} else
		if (layer.getExtrudingDirection()==2) {
			point.addContent("0.0,0.0,"+layer.getExtrusionStop());
		}
		vector.addContent(point);
		child.addContent(vector);
	
		return child;
	}

	private String selectFile(String path)
	{
		JFileChooser fc = new JFileChooser();
		fc.setCurrentDirectory(new File(path));
		File file = null;
		String fileName = null;
		
		int retVal = fc.showOpenDialog( null );

		if(retVal==JFileChooser.APPROVE_OPTION ) {
			file = fc.getSelectedFile();
			fileName = file.getAbsolutePath(); // getName();
			currentFilePath = file.getPath();
			System.out.println( file.getAbsolutePath().replaceAll(file.getName(), "") );
		}
		else
			System.out.println( "Auswahl abgebrochen" );	
		
		return fileName;
	}

	public String getCurrentFilePath() {
		return currentFilePath;
	}
	public void setCurrentFilePath(String currentFilePath) {
		this.currentFilePath = currentFilePath;
	}
}
