/*
Class-name: Layer.java, part of the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

package interfaces;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Shape;
import java.util.Vector;

public class Layer implements CadObject
{
	private String name;
	private int layerID;
	private double lineWidth = 1d;
	private Color lineColor = Color.BLACK;
	private Color fillColor = Color.YELLOW;
	private int lineTyp = 0;
	private BasicStroke basicStroke = new BasicStroke(1.0f);
	private int[] usedObjects = null;
	private int extrudingDirection;
	private double extrusionStart = 0d, extrusionStop = 0d;
	private boolean visible = true;
	private boolean folderOpen = true;
	private double transparency = 1d;
	private Vector<LayerProperty> layerProperties = new Vector<LayerProperty>();

	/**
	 * Konstructors
	 */
	public Layer()
	{
		initLayer("Default-Layer", 0);
	}
	public Layer(String name, int layerID)
	{
		initLayer(name, layerID);
	}

	private void initLayer(String name, int layerID)
	{
		this.name = name;
		this.layerID = layerID;
	}
	
	/**
	 * Adding a new layer-property
	 *
	 */
	public void addNewProperty(String propertyName, String propertyValue)
	{
		layerProperties.add(new LayerProperty(propertyName, propertyValue));
	}
	public void removeProperty(int id)
	{
		layerProperties.remove(id);
	}
	
	public BasicStroke getBasicStroke()
	{
		return basicStroke;
	}
	public void setBasicStroke(BasicStroke basicStroke)
	{
		this.basicStroke = basicStroke;
	}
	public Color getFillColor()
	{
		return fillColor;
	}
	public void setFillColor(Color fillColor)
	{
		this.fillColor = fillColor;
	}
	public Color getLineColor()
	{
		return lineColor;
	}
	public void setLineColor(Color lineColor)
	{
		this.lineColor = lineColor;
	}
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	public int[] getUsedObjects()
	{
		return usedObjects;
	}
	public void setUsedObjects(int[] usingObjects)
	{
		this.usedObjects = usingObjects;
	}
	public int getLayerID() {
		return layerID;
	}
	public void setLayerID(int layerID) {
		this.layerID = layerID;
	}
	public double getLineWidth() {
		return lineWidth;
	}
	public void setLineWidth(double lineWidth) {
		this.lineWidth = lineWidth;
	}
	public boolean isVisible() {
		return visible;
	}
	public void setVisible(boolean visible) {
		this.visible = visible;
	}
	public int getLineTyp() {
		return lineTyp;
	}
	public void setLineTyp(int lineTyp) {
		this.lineTyp = lineTyp;
	}
	public double getTransparency() {
		return transparency;
	}
	public void setTransparency(double transparency) {
		this.transparency = transparency;
	}
	public boolean isFolderOpen() {
		return folderOpen;
	}
	public void setFolderOpen(boolean folderOpen) {
		this.folderOpen = folderOpen;
	}
	public Vector<LayerProperty> getLayerProperties() {
		return layerProperties;
	}
	public void setLayerProperties(Vector<LayerProperty> layerProperties) {
		this.layerProperties = layerProperties;
	}
	public int getExtrudingDirection() {
		return extrudingDirection;
	}
	public void setExtrudingDirection(int extrudingDirection) {
		this.extrudingDirection = extrudingDirection;
	}
	public double getExtrusionStart() {
		return extrusionStart;
	}
	public void setExtrusionStart(double extrusionStart) {
		this.extrusionStart = extrusionStart;
	}
	public double getExtrusionStop() {
		return extrusionStop;
	}
	public void setExtrusionStop(double extrusionStop) {
		this.extrusionStop = extrusionStop;
	}
	
	// Unused methods of the CadObject-interface
	public int getGeometryTyp() {
		// TODO Auto-generated method stub
		return 0;
	}
	public int getNumberOfPoints() {
		// TODO Auto-generated method stub
		return 0;
	}
	public int getObjectId() {
		// TODO Auto-generated method stub
		return 0;
	}
	public Shape getShape(int panelID) {
		// TODO Auto-generated method stub
		return null;
	}
	public Shape getShape(int panelID, double scale) {
		// TODO Auto-generated method stub
		return null;
	}
	public Vector<Vertex> getVertexList() {
		// TODO Auto-generated method stub
		return null;
	}
	public boolean isFillable() {
		// TODO Auto-generated method stub
		return false;
	}
	public boolean isFilled() {
		// TODO Auto-generated method stub
		return false;
	}
	public boolean isInBuildProcess() {
		// TODO Auto-generated method stub
		return false;
	}
	public boolean isNewObject() {
		// TODO Auto-generated method stub
		return false;
	}
	public void setFillable(boolean visible) {
		// TODO Auto-generated method stub
	}
	public void setFilled(boolean filled) {
		// TODO Auto-generated method stub
	}
	public void setGeometryTyp(int geometryTyp) {
		// TODO Auto-generated method stub
	}
	public void setInBuildProcess(boolean moving) {
		// TODO Auto-generated method stub
	}
	public void setNewObject(boolean newObject) {
		// TODO Auto-generated method stub
	}
	public void setNumberOfPoints(int numberOfPoints) {
		// TODO Auto-generated method stub
	}
	public void setObjectId(int id) {
		// TODO Auto-generated method stub
	}
	public void setObjectPoints(int pointNumber, double x1, double y1, double z1, double x2, double y2, double z2, double scale) {
		// TODO Auto-generated method stub
	}
	public void setShape(Shape shape) {
		// TODO Auto-generated method stub
	}
	public void setVertexList(Vector<Vertex> vertexList) {
		// TODO Auto-generated method stub
	}
	public void finalizeObject(int quadrant) {
		// TODO Auto-generated method stub		
	}
	public int getCurrentPointNumber() {
		// TODO Auto-generated method stub
		return 0;
	}
	public void setCurrentPointNumber(int currentPointNumber) {
		// TODO Auto-generated method stub
	}
	public boolean isRendered() {
		// TODO Auto-generated method stub
		return false;
	}
	public void setRendered(boolean rendered) {
		// TODO Auto-generated method stub
	}
	public boolean isRendered2D() {
		// TODO Auto-generated method stub
		return false;
	}
	public void setRendered2D(boolean rendered) {
		// TODO Auto-generated method stub
	}
	public Shape getShapeXY() {
		// TODO Auto-generated method stub
		return null;
	}
	public Shape getShapeXZ() {
		// TODO Auto-generated method stub
		return null;
	}
	public Shape getShapeZY() {
		// TODO Auto-generated method stub
		return null;
	}
	public Shape initShape(int panelID, double scale) {
		// TODO Auto-generated method stub
		return null;
	}
	public int getBuiltInView() {
		// TODO Auto-generated method stub
		return 0;
	}
	public void setBuiltInView(int builtInView) {
		// TODO Auto-generated method stub
		
	}
	// Unused methods of the CadObject-interface - END
	
	public class LayerProperty
	{
		protected String propertyName = null;
		protected String propertyValue = null;
		
		public LayerProperty(String propertyName, String propertyValue)
		{
			this.propertyName = propertyName;
			this.propertyValue = propertyValue;
		}
		
		public String getPropertyName() {
			return propertyName;
		}
		public void setPropertyName(String propertyName) {
			this.propertyName = propertyName;
		}
		public String getPropertyValue() {
			return propertyValue;
		}
		public void setPropertyValue(String propertyValue) {
			this.propertyValue = propertyValue;
		}
	}
}
